# NGSpice Benchmark Script
#
# Udo Krebelder <udo.krebelder@gmx.at> 09/04/2025
#

$ErrorActionPreference = "Stop"

# List of NGSpice test files
$files = @( 'mux8.ngcir',
            'mike2.ngcir',
            'bjtff.ngcir',
            'AzurAmp2.ngcir',
            'OscillatorQuadrature2.ngcir'
            )

# NGSpice executable, user defined location
#$NGSpice = "Set to your preferred NGSpice version"
$NGSpice = "C:\msys64\mingw64\bin\ngspice_con.exe"

if (!(Test-Path $NGSpice)) {
    Write-Error "Cannot find NGSpice executable" -ErrorAction Stop
}

#Write-Output ('NGSpice: ' + $NGSpice)

$ver = [Diagnostics.FileVersionInfo]::GetVersionInfo($NGSpice)

$date = date
Write-Output ("$NGSpice Benchmark")
Write-Output ''
Write-Output ("$ver.ProductVersion")
Write-Output ''
Write-Output 'CPU Info...'
$proc = Get-WmiObject -Class Win32_Processor | Select-Object -Property Name, NumberOfCores, NumberOfLogicalProcessors
($proc | Out-String).Trim() | Write-Output
Write-Output ''

$runs = @()
$totaltime = 0

foreach($file in $files)
{
    Write-Output ('Running ' + $file + ': ')

    $b = (Get-Item $file).basename
    $l = $b + '.log'
    $ngr = $b + '.ngraw'
    if (Test-Path $l) { Remove-Item $l }
    if (Test-Path $ngr) { Remove-Item $ngr }

    # time2 is the execution time
    #Start-Process -FilePath $NGSpice -Wait -ArgumentList "-b -run $file"
    #$time2 = $sw.Elapsed
    #$time2 = (Measure-Command -Expression { Start-Process -NoNewWindow -Wait -FilePath $NGSpice -ArgumentList "$file" } ).TotalSeconds

    Start-Process -NoNewWindow -Wait -FilePath $NGSpice -ArgumentList "-b $file -o $l -r $ngr"
    if (Test-Path $ngr) { Remove-Item $ngr }

    # time is the simulation time from the log file
    $f = (Select-String -path $l -pattern "Total elapsed time")
    $a = @($f -split ' ')
    $time = $a[5]


    #Write-Output ('x11: ' + $f + 'x99')
    #Write-Output ('x10: ' + $time)

    $runs += [pscustomobject]@{'File'=$file;'ExecTime'=[math]::Round($time,1)}

    $totaltime += $time
    Write-Output ('  ' + [math]::Round($time,1) + ' seconds')
    Write-Output ''
    Start-Sleep -m 100

    $f = $b + '.qraw'
    if (Test-Path $f) { Remove-Item $f }

    $f = $b + '.op.qraw'
    if (Test-Path $f) { Remove-Item $f }
}

Write-Output ('Total time: ' + [math]::Round($totaltime,0) + ' seconds')
Write-Output ''

$logfile = '7_NGSpiceBenchmarkReport.txt'

if (!(Test-Path $logfile)) { New-Item -Path . -Name $logfile -ItemType "file" }

Add-Content -Path $logfile '********************************************************************************'
Add-Content -Path $logfile ("NGSpice " + $ver.ProductVersion + " Benchmark")
Add-Content -Path $logfile ($env:computername + ' ' + $date)
Add-Content -Path $logfile ''

($proc | Out-String).Trim() | Out-File $logfile -Append -Encoding ASCII
Add-Content -Path $logfile ''

($runs | Select-Object -Property File,ExecTime | Out-String).Trim() | Out-File $logfile -Append -Encoding ASCII
Add-Content -Path $logfile ''

Add-Content -Path $logfile ('Total time: ' + [math]::Round($totaltime,0) + ' seconds')
Add-Content -Path $logfile ''
Add-Content -Path $logfile ''

#pause
